﻿using System;
using System.Collections.Generic;
using CevoAILib;
using CevoAILib.Diplomacy;

// This is version 1.0.5 from 02.02.2019

    

namespace AI
{		
    
	sealed class Empire : AEmpire
	{
		// Define Settler instances
		SettlerAdmin AdminSettlers = new SettlerAdmin();
		Settler[] Settlers = new Settler[256];

        // Define  Military instances
        Military[] Militaries = new Military[1024];
        AdminMilitary AdminMilitaries = new AdminMilitary();

        // Define City instances
        CityAdmin AdminCities = new CityAdmin();
		XCity[] XCities = new XCity[256];
			
		// Define GridMap
		GridMap XGridMap = new GridMap();
	
		// Random Generator
		Random random = new Random();

        // Store City Location
        Location[] CityLocs = new Location[256];

        // Research Plan
        ResearchFunctions ResearchFunction = new ResearchFunctions();

        // Building Plan
        BuildingFunctions BuildingFunction = new BuildingFunctions();

        // Initialize Economy
        Economy economy = new Economy();
        
        

        // City and Unit control
        // Settler Control
        int oldSettlerNum = 1;
        int newSettlerNum = 1;
		int assignedSettlerIndex = 0;
		int countSettlers = 0;
        bool foundUnitID = false;

        // Military Control
        int oldMilitaryNum = 0;
        int newAssignedMilitaryNum = 0;
        int totalMilitaryNum = 0;
        bool finishedAllMilitaryUnits = false;
        int assignedMilitaryIndex = 0;
        int countMilitaries = 0;

        // City Contol
        int oldCityNum = 1;
        int newCityNum = 1;
        int assignedCityIndex = 0;
		int countCities = 0;
        bool foundCityID = false;
        bool resumeBuilds = false;

        // Status Variables for Control
        int CityCount = 0;
		bool newSettler;
        bool newMilitary;
        bool newCity;

        int ResearchCount = 0;

        public Empire(int nationID, IntPtr serverPtr, IntPtr dataPtr, bool isNewGame)
			: base(nationID, serverPtr, dataPtr, isNewGame)
		{
		}
		
		protected override void NewGame()
		{
		}


		protected override void Resume()
		{
		}

        protected override void OnTurn()
        {
            // set TaxRate
            int money = this.Treasury;
            if (money < 500)
            {
                Economy economy = new Economy(100, 0);
            }
            else if (money < 1100)
            {
                Economy economy = new Economy(90, 0);
            }
            else if (money < 2500)
            {
                Economy economy = new Economy(75, 0);
            }
            else if (money < 3500)
            {
                Economy economy = new Economy(50, 0);
            }
            else if (money < 4500)
            {
                Economy economy = new Economy(42, 0);
            }
            ChangeEconomy__Turn(economy);

            // Initialize Research-Plan
            if (Turn == 0)
            {
                ResearchFunction.AssignFirstResearchFunctions();
                BuildingFunction.AssignFirstBuildingFunctions();
            }

            // check if research complete
            if (HadEvent__Turn(EmpireEvent.ResearchComplete))
            {
                if (ResearchFunction.getResearchPlan(ResearchCount) != Advance.None)
                {
                    SetResearch__Turn(ResearchFunction.getResearchPlan(ResearchCount));
                    ResearchCount++;
                }

                // Pikenier
                if ((ResearchCount == 4) && (ResearchFunction.getResearchPlan(ResearchCount) != Advance.None))
                {
                    Blueprint.SetDomain__Turn(ModelDomain.Ground);
                    Blueprint.SetProperty__Turn(ModelProperty.Weapons, 3);
                    Blueprint.SetProperty__Turn(ModelProperty.Armor, 2);
                    Blueprint.SetProperty__Turn(ModelProperty.Mobility, Blueprint.Stage.MaximumWeight - Blueprint.Weight);
                    SetResearch__Turn(Advance.MilitaryResearch);
                }

                // Dragoons
                if ((ResearchCount == 19) && (ResearchFunction.getResearchPlan(ResearchCount) != Advance.None))
                {
                    Blueprint.SetDomain__Turn(ModelDomain.Ground);
                    Blueprint.SetProperty__Turn(ModelProperty.Weapons, 1);
                    Blueprint.SetProperty__Turn(ModelProperty.Armor, 2);
                    Blueprint.SetProperty__Turn(ModelProperty.Mobility, Blueprint.Stage.MaximumWeight - Blueprint.Weight);
                    SetResearch__Turn(Advance.MilitaryResearch);
                }
            }

            // Set Anarchy and then Monarchy
            if ((ResearchCount == 8) && (Government == Government.Despotism))
            {
                Revolution__Turn();
            }
            if (HadEvent__Turn(EmpireEvent.AnarchyOver))
            {
                SetGovernment__Turn(Government.Monarchy);
                resumeBuilds = true;
            }

            // Initialize model Settler	and Military Units
            Model modelSettlers = new Model(this, 0);
            Model militia = new Model(this, 1);
            Model townguard = new Model(this, 2);
            Model pikenier = new Model(this, 3);
            Model dragoons = new Model(this, 4);

            // Initialize first Settler and City
            if (Turn == 0)
            {
                foreach (Unit unit in Units) Settlers[0] = new Settler(unit.ID, 1);
                AdminSettlers.XSettlerJob[0] = SettlerToDo.BuildIrrigation;
                Settlers[0].settlerHasOrders = true;
                foreach (City city in Cities) XCities[0] = new XCity(city.ID, city.Location);
            }


            // Initialize first GridMap instances and Location of Home City
            if (Turn == 0)
            {
                XGridMap = new GridMap();
                for (int i = 0; i < 9; i++) XGridMap.GridMapEntity[i] = new GridMapEntity();
                foreach (City city in Cities) CityLocs[0] = city.Location;
            }


            // Check for new Cities
            foreach (City city in Cities)
            {
                newCity = true;
                for (int i = 0; i < oldCityNum; i++)
                {
                    if (XCities[i].cID == city.ID) newCity = false;
                }

                if (newCity == true)
                {
                    XCities[newCityNum] = new XCity(city.ID, city.Location);
                    AdminCities.numCities++;
                    newCityNum++;
                    CityLocs[newCityNum - 1] = city.Location;
                }
            }
            oldCityNum = newCityNum;

            // Check for new Settler units			
            foreach (Unit unit in Units)
            {
                if (unit.Model.Kind == ModelKind.Settlers)
                {
                    newSettler = true;
                    for (int i = 0; i < oldSettlerNum; i++)
                    {
                        if (Settlers[i].uID == unit.ID) newSettler = false;
                    }

                    if (newSettler == true)
                    {
                        Settlers[newSettlerNum] = new Settler(unit.ID, newSettlerNum + 1);
                        newSettlerNum++;
                    }
                }                
            }
            oldSettlerNum = newSettlerNum;

            // Check for new Military units			
            foreach (Unit unit in Units)
            { 
                if ((unit.Model.Kind == ModelKind.OwnDesign) && (totalMilitaryNum >= 1))
                {
                    newMilitary = true;
                    
                    for (int i = 0; i <= (totalMilitaryNum-1); i++)
                    {
                        if (Militaries[i].mID == unit.ID) newMilitary = false;
                    }

                    if (newMilitary == true)
                    {
                        Militaries[totalMilitaryNum-1].mID = unit.ID;
                    }
                }
            }


            if (oldSettlerNum == 7)
            {
                // test
            }          

			// Initializing City Grid Map for Home City
			if (Turn == 0)
			{
				foreach (City city in Cities)
				{
					XGridMap.GridMapEntity[0].GridLocation = city.Location;
					XGridMap.AssignFirstGridMapLocs(city.Location);
				}
			}

            

            // Start City commands
            foreach (City city in Cities)
            {
                // Retrieve corresponding City ID
                countCities = 0;
                foundCityID = false;
                do
                {
                    if (city.ID != XCities[countCities].cID) countCities += 1;
                    else
                    {
                        assignedCityIndex = countCities;
                        foundCityID = true;
                        countCities++;
                    }
                } while ((foundCityID == false) && (countCities != newCityNum));

                // Issue City Commands
                // Orders for Home City
                if (assignedCityIndex == 0)
                {
                    if ((city.Size >= 5) && (oldSettlerNum < 9))
                    {
                        city.OptimizeExploitedLocations__Turn(ResourceWeights.HurryProduction);
                        city.SetUnitInProduction__Turn(modelSettlers, UnitProductionOptions.None);
                    }
                    else
                    {          
                        city.OptimizeExploitedLocations__Turn(ResourceWeights.MaxGrowth);
                        city.StopProduction__Turn();
                    }
                }

                // Orders for further Cities
                else if (assignedCityIndex > 0)
                {
                    
                    if (city.Size < 12)
                        city.OptimizeExploitedLocations__Turn(ResourceWeights.MaxGrowth);
                    else
                        city.OptimizeExploitedLocations__Turn(ResourceWeights.HurryProduction);

                    // Build the first four buildings: Barracks, Temple, City Walls, Aqueduct
                    if ((ResearchCount >= (BuildingFunction.assignedResearch(XCities[assignedCityIndex].buildingCount) + 1)) && (XCities[assignedCityIndex].buildingCount <= 3))
                    {                       
                        if (city.HadEvent__Turn(CityEvent.ProductionComplete))
                        {
                            XCities[assignedCityIndex].productionComplete = true;
                            XCities[assignedCityIndex].buildingCount += 1;
                        }

                        if (((XCities[assignedCityIndex].productionComplete) ||
                            (XCities[assignedCityIndex].buildingCount == 0) || (resumeBuilds == true)) &&
                            (Government != Government.Anarchy) &&
                            (XCities[assignedCityIndex].buildingCount <= 3))
                        {
                            if (money >= 300)
                            {
                                city.SetBuildingInProduction__Turn(BuildingFunction.getBuildingPlan(XCities[assignedCityIndex].buildingCount));
                                XCities[assignedCityIndex].productionComplete = false;
                            }
                        }                          
                    }

                    // Build 2 pikeniers in every city for protection
                    else if ((XCities[assignedCityIndex].buildingCount == 4) && (XCities[assignedCityIndex].newMilitaryNum <= 1))
                    {
                        if ((city.HadEvent__Turn(CityEvent.ProductionComplete)) &&
                        (XCities[assignedCityIndex].militaryTrigger == true))
                        {
                            XCities[assignedCityIndex].productionComplete = true;
                            XCities[assignedCityIndex].unitInProduction = false;
                            XCities[assignedCityIndex].newMilitaryNum++;
                            Militaries[totalMilitaryNum]= new Military("pikenier");
                            totalMilitaryNum++;

                            if (XCities[assignedCityIndex].newMilitaryNum == 2)
                            {
                                city.StopProduction__Turn();
                                XCities[assignedCityIndex].militaryTrigger = false;
                                XCities[assignedCityIndex].unitInProduction = false;
                            }
                        }
                        XCities[assignedCityIndex].militaryTrigger = true;

                        if ((((XCities[assignedCityIndex].productionComplete) && (XCities[assignedCityIndex].newMilitaryNum <= 1))
                            || ((XCities[assignedCityIndex].newMilitaryNum == 0)) && (XCities[assignedCityIndex].unitInProduction == false)))
                        {
                            city.SetUnitInProduction__Turn(pikenier, UnitProductionOptions.None);
                            XCities[assignedCityIndex].productionComplete = false;
                            XCities[assignedCityIndex].unitInProduction = true;
                        }
                    }
                    // Build the Theater
                    else if (ResearchCount >= (BuildingFunction.assignedResearch(XCities[assignedCityIndex].buildingCount)))
                    {
                        if ((XCities[assignedCityIndex].buildingCount >= 3) && (XCities[assignedCityIndex].newMilitaryNum == 2))
                        {
                            if ((city.HadEvent__Turn(CityEvent.ProductionComplete)) && (XCities[assignedCityIndex].buildingTrigger == true))
                            {
                                XCities[assignedCityIndex].productionComplete = true;
                                XCities[assignedCityIndex].buildingCount += 1;
                            }
                            XCities[assignedCityIndex].buildingTrigger = true;

                            if ((XCities[assignedCityIndex].productionComplete) &&
                               (XCities[assignedCityIndex].buildingCount == 4) && (resumeBuilds == true))
                            {
                                if (money >= 200)
                                {
                                    city.SetBuildingInProduction__Turn(BuildingFunction.getBuildingPlan(XCities[assignedCityIndex].buildingCount));
                                    XCities[assignedCityIndex].productionComplete = false;
                                }
                            }
                        }
                    }

                    // newMilitaryNum == 2, Buildingcount == 5
                    // Build the dragoons
                    if ((XCities[assignedCityIndex].buildingCount == 5) && (XCities[assignedCityIndex].newMilitaryNum <= 6) && (ResearchCount >= 21))
                    {
                        if (city.HadEvent__Turn(CityEvent.ProductionComplete) ||
                           ((XCities[assignedCityIndex].newMilitaryNum == 2) && (XCities[assignedCityIndex].militaryTrigger == true)))
                        {
                            XCities[assignedCityIndex].productionComplete = true;
                            XCities[assignedCityIndex].unitInProduction = false;
                            XCities[assignedCityIndex].newMilitaryNum++;
                            Militaries[totalMilitaryNum] = new Military("dragoon");
                            totalMilitaryNum++;

                            if (XCities[assignedCityIndex].newMilitaryNum == 7)
                            {
                                city.StopProduction__Turn();
                                XCities[assignedCityIndex].militaryTrigger = false;
                                XCities[assignedCityIndex].unitInProduction = false;
                                XCities[assignedCityIndex].hasAllMilitaryUnits = true;
                            }
                        }
                        XCities[assignedCityIndex].militaryTrigger = true;

                        if ((XCities[assignedCityIndex].productionComplete) && (XCities[assignedCityIndex].newMilitaryNum <= 6)
                             && (XCities[assignedCityIndex].unitInProduction == false))
                        {
                            city.SetUnitInProduction__Turn(dragoons, UnitProductionOptions.None);
                            XCities[assignedCityIndex].productionComplete = false;
                            XCities[assignedCityIndex].unitInProduction = true;
                        }
                    }
                }
            }
            // current counter: newMilitaryNum == 7, Buildingcount == 5

            // quest if all Military Units are ready for Attack
            finishedAllMilitaryUnits = true;
            for (int i = 0; i < countCities; i++)
            {
                if (XCities[assignedCityIndex].hasAllMilitaryUnits == false)
                {
                    finishedAllMilitaryUnits = false;
                }
            }
            

            // Start Settler and Military commands
            foreach (Unit unit in Units)
            {
                if (unit.Model.Kind == ModelKind.Settlers)
                {
                    // Retrieve corresponding Settler ID
                    countSettlers = 0;
                    foundUnitID = false;
                    do
                    {
                        if (unit.ID != Settlers[countSettlers].uID) countSettlers++;
                        else
                        {
                            assignedSettlerIndex = countSettlers;
                            foundUnitID = true;
                            countSettlers++;
                        }
                    } while ((foundUnitID == false) && (countSettlers != newSettlerNum));

                    // Define City
                    CityCount = 0;

                    // Issue Settler Command for Settlers 1 to 8
                    if (assignedSettlerIndex <= 7)
                    {
                        if (Settlers[assignedSettlerIndex].settlerHasOrders == true)
                        {
                            // Do nothing
                        }
                        else
                        {
                            AdminSettlers.XSettlerJob[assignedSettlerIndex] = SettlerToDo.SeekNewSettleLocation;
                            Settlers[assignedSettlerIndex].settlerHasOrders = true;
                        }

                        RC RelCoor = unit.Location - XCities[CityCount].cityLocation;
                        AdminSettlers.commandSettlerMove(unit, RelCoor, Settlers[assignedSettlerIndex],
                                                         XCities, AdminCities.numCities, CityCount, oldCityNum, 
                                                         assignedSettlerIndex, XGridMap, CityLocs);
                    }
                    else
                    {
                        if (Settlers[assignedSettlerIndex].settlerHasOrders == true)
                        {
                            // Do nothing
                        }
                        else
                        {
                            AdminSettlers.XSettlerJob[assignedSettlerIndex] = SettlerToDo.MoveToCity;
                            Settlers[assignedSettlerIndex].settlerHasOrders = true;
                        }

                        RC RelCoor = unit.Location - XCities[AdminSettlers.numMoveToCity].cityLocation;
                        AdminSettlers.commandSettlerMove(unit, RelCoor, Settlers[assignedSettlerIndex],
                                                         XCities, AdminCities.numCities, AdminSettlers.numMoveToCity, oldCityNum, 
                                                         assignedSettlerIndex, XGridMap, CityLocs);
                    }
                }
                else if (unit.Model.Kind == ModelKind.OwnDesign)
                {
                    newMilitary = true;
                    for (int i = 0; i < oldMilitaryNum; i++)
                    {
                        if (Militaries[i].mID == unit.ID) newMilitary = false;
                    }

                    if (newMilitary == true)
                    {
                        Militaries[newAssignedMilitaryNum].mID = unit.ID;
                        newAssignedMilitaryNum++;
                    }
                    oldMilitaryNum = newAssignedMilitaryNum;
                    

                    if (finishedAllMilitaryUnits == true)
                    {
                        countMilitaries = 0;
                        foundUnitID = false;
                        do
                        {
                            if (unit.ID != Militaries[countMilitaries].mID) countMilitaries++;
                            else
                            {
                                assignedMilitaryIndex = countMilitaries;
                                foundUnitID = true;
                                countMilitaries++;
                            }
                        } while ((foundUnitID == false) && (countMilitaries != totalMilitaryNum));

                        if (Militaries[assignedMilitaryIndex].type == "pikenier")
                        {
                            AdminMilitaries.moveToAttack(unit, Militaries[assignedMilitaryIndex].mID);
                        }
                    }
                }
            }
		}

		protected override void OnStealAdvance(Advance[] selection)
		{
			StealAdvance__Turn(selection[random.Next(selection.Length)]);
		}

		protected override void OnChanceToNegotiate(Phase situation, Nation Opponent, ref bool wantNegotiation, ref bool cancelTreatyIfRejected)
		{
			if (situation == Phase.BeginOfTurn) // start negotiation?
			{
				if (random.Next(30) == 0)
					wantNegotiation = true;
			}
			if (situation == Phase.ForeignTurn) // accept contact for negotiation?
			{
				if (random.Next(2) == 0)
					wantNegotiation = true;
			}
		}

		protected override void OnNegotiate(Negotiation negotiation)
		{
			if (negotiation.History.Count == 0 && RelationTo(negotiation.Opponent) < Relation.Alliance)
				negotiation.SetOurNextStatement(new SuggestTrade(new ITradeItem[] { new ChangeRelation(RelationTo(negotiation.Opponent) + 1) }, null)); // suggest better treaty
			if (negotiation.History.Count > 0 && negotiation.History[0].OpponentResponse is SuggestTrade)
			{
				SuggestTrade trade = negotiation.History[0].OpponentResponse as SuggestTrade;
				if ((trade.Offers.Length == 1 && trade.Wants.Length == 0 && trade.Offers[0] is ChangeRelation &&
					((ChangeRelation)trade.Offers[0]).NewRelation > RelationTo(negotiation.Opponent) ||
					(trade.Offers.Length == 0 && trade.Wants.Length == 1 && trade.Wants[0] is ChangeRelation &&
					((ChangeRelation)trade.Wants[0]).NewRelation > RelationTo(negotiation.Opponent))))
					negotiation.SetOurNextStatement(new AcceptTrade()); // accept better treaty
			}
		}

		protected override void OnForeignMove(IUnitInfo unit, Location destination)
		{
		}

		protected override void OnBeforeForeignAttack(IUnitInfo attacker, Location target, BattleOutcome outcome)
		{
		}

		protected override void OnAfterForeignAttack()
		{
		}

		protected override void OnBeforeForeignCapture(Nation nation, ICity city)
		{
		}

		protected override void OnAfterForeignCapture()
		{
		}
	}
}
